/*! \file enc28j60.c \brief Microchip ENC28J60 Ethernet Interface Driver. */
// *****************************************************************************
//
// File Name	: 'enc28j60.c'
// Title		: Microchip ENC28J60 Ethernet Interface Driver
// Author		: Pascal Stang (c)2005
// Created		: 9/22/2005
// Revised		: 9/22/2005
// Version		: 0.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
// Description	: This driver provides initialization and transmit/receive
//	functions for the Microchip ENC28J60 10Mb Ethernet Controller and PHY.
// This chip is novel in that it is a full MAC+PHY interface all in a 28-pin
// chip, using an SPI interface to the host processor.
//
// *****************************************************************************

//#include "timer.h"
//#include "rprintf.h"

#include "ENC28J60.h"
#include <avr/io.h>
// global AVRLIB defines
#include "avrlibdefs.h"
#include "uart.h"
#include "Stack.h"
#include <avr/pgmspace.h>


Byte Enc28j60Bank;
Word NextPacketPtr;
#define ENC28J60_CONTROL_PORT	PORTB
#define ENC28J60_CONTROL_DDR	DDRB
#define ENC28J60_CONTROL_CS	   2

volatile Boolean ENC28J60_Interrupt0;
Word packet_lenght;

//#define F_CPU        	8000000               		/* Processor Speed */
#define CYCLES_PER_US 	((F_CPU+500000)/1000000) 	/* cpu cycles per microsecond */

void delay(Word us) 
{
    Word  delay_loops;
    register Word  i;

    delay_loops = (us+3)/5*CYCLES_PER_US; // +3 for rounding up (dirty) 

	// one loop takes 5 cpu cycles 
    for (i=0; i < delay_loops; i++) {};
} 

void nicSetMacAddress(Byte* macaddr)
{
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, *macaddr++);
   	enc28j60Write(MAADR4, *macaddr++);
	enc28j60Write(MAADR3, *macaddr++);
	enc28j60Write(MAADR2, *macaddr++);
	enc28j60Write(MAADR1, *macaddr++);
	enc28j60Write(MAADR0, *macaddr++);
}





Byte enc28j60ReadOp(Byte op, Byte address)
{
	Byte data;
   
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue read command
   	SPDR = op | (address & ADDR_MASK);
	while(!(SPSR & (1<<SPIF)));
	// read data
   	SPDR = 0x00;
   	while(!(SPSR & (1<<SPIF)));
	// do dummy read if needed
   	if(address & 0x80)
	{
		SPDR = 0x00;
   	while(!(inb(SPSR) & (1<<SPIF)));
	}
	data = SPDR;
	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);

	return data;
}

void enc28j60WriteOp(Byte op, Byte address, Byte data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);

	// issue write command
	SPDR = op | (address & ADDR_MASK);
	while(!(SPSR & (1<<SPIF)));
	// write data
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));

	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60ReadBuffer(Word len, Byte* data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue read command
	SPDR = ENC28J60_READ_BUF_MEM;
	while(!(SPSR & (1<<SPIF)));
	while(len--)
	{
		// read data
		SPDR = 0x00;
		while(!(SPSR & (1<<SPIF)));
		*data++ = SPDR;
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60WriteBuffer(Word len, Byte* data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue write command
	SPDR = ENC28J60_WRITE_BUF_MEM;
	while(!(SPSR & (1<<SPIF)));
	while(len--)
	{
		// write data
		SPDR = *data++;
		while(!(SPSR & (1<<SPIF)));
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60SetBank(Byte address)
{
	// set the bank (if needed)
	if((address & BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		Enc28j60Bank = (address & BANK_MASK);
	}
}

Byte enc28j60Read(Byte address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

void enc28j60Write(Byte address, Byte data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}



void enc28j60PhyWrite(Byte address, Word data)
{
	// set the PHY register address
	enc28j60Write(MIREGADR, address);
	
	// write the PHY data
	enc28j60Write(MIWRL, data);	
	enc28j60Write(MIWRH, data>>8);

	// wait until the PHY write completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);
}

void enc28j60Init(void)
{
   ENC28J60_Interrupt0=false;


	// initialize I/O

	ENC28J60_CONTROL_DDR |= 1<<ENC28J60_CONTROL_CS;
	ENC28J60_CONTROL_PORT |= 1<<ENC28J60_CONTROL_CS;
	

	
	PORTB |= 1<<PB5; //SCK HI
	DDRB  |= 1<<PB2 |1<<PB3 | 1<<PB5; // mosi, sck, ss output
	DDRB  &= ~(1<<PB4); // miso input
	
	// initialize SPI interface
	// master mode und /2x takt (klappt auch wenn der mega8 mit 16mhz luft !!)
	SPCR |= 1<<MSTR | 1<<SPE;
	SPSR |= 1<<SPI2X;
	

	// perform system reset
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	// check CLKRDY bit to see if reset is complete
	delay(50);
	while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY));

	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	NextPacketPtr = RXSTART_INIT;
	enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXSTH, RXSTART_INIT>>8);
	// set receive pointer address
	enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);
	// set receive buffer end
	// ERXND defaults to 0x1FFF (end of ram)
	enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);
	enc28j60Write(ERXNDH, RXSTOP_INIT>>8);
	// set transmit buffer start
	// ETXST defaults to 0x0000 (beginnging of ram)
	enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);
	enc28j60Write(ETXSTH, TXSTART_INIT>>8);

	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(MACON2, 0x00);
	// enable automatic padding and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);

	// set inter-frame gap (non-back-to-back)
	enc28j60Write(MAIPGL, 0x12);
	enc28j60Write(MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);	
	enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);

	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, ENC28J60_MAC0);
	enc28j60Write(MAADR4, ENC28J60_MAC1);
	enc28j60Write(MAADR3, ENC28J60_MAC2);
	enc28j60Write(MAADR2, ENC28J60_MAC3);
	enc28j60Write(MAADR1, ENC28J60_MAC4);
	enc28j60Write(MAADR0, ENC28J60_MAC5);

	// no loopback of transmitted frames
	enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);

	// switch to bank 0
	enc28j60SetBank(ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);
	//enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, 0xFF);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);

   // Interupt Handler eintragen
   // INT0 PIN als Eingang
   ClearBit(ENC28J60_DDR,ENC28J60_PIN);

	//Mac Adresse setzen(stack.h, dort wird auch die Ip festgelegt)
   nicSetMacAddress(mymac);
	
	/*	Leds konfigurieren
		LEDA : Link status
		LEDB : Receive activity
		Led Settings : enc28j60 datasheet, page 11*/
	
	//enc28j60PhyWrite(0x14,0b0000010000100000);

	enc28j60PhyWrite(0x14,0b0011011101001010);
	
	/*	Leds konfigurieren
		LEDA : Link status
		LEDB : Blink Fast
		Led Settings : enc28j60 datasheet, page 11
		Auskommentieren, wenn mans lieber blinken sieht ;-)		*/
	
	//enc28j60PhyWrite(0x14,0b0000010000100000);

   cli();
   EICRA = (1<<ISC01) + (0<<ISC00); // INT0 Falling edge
   //EICRA |= (0<<ISC01) + (0<<ISC00); // INT0 Low level
   EIMSK |= (1<<INT0 ); // INT0 aktiviert
   sei();

}

ISR (INT0_vect)
{
   // Interuptstatus abfragen

   //enc28j60ReadOp(Byte op, Byte address)

   //T_EIR______Ethernet_Interrupt_Request_Flag_Register EIR;
   //EIR.byte = ENC28J60_Read_Control_Register_8_Bit(EIR______Ethernet_Interrupt_Request_Flag_Register);

   // Interuptstatus zurcksetzen
   //ENC28J60_Write_Control_Register_8_Bit(EIR______Ethernet_Interrupt_Request_Flag_Register,0);

   enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, EIR, EIE_INTIE|EIE_PKTIE);

//   uart_puts("EIR = ");
/*   char Buffer[5];
   IntToStr(EIR.byte,Buffer);
   uart_puts(Buffer);*/
//   uart_putc(0x0D);
//   uart_putc(0x0A);

   // intere Abarbeitung beauftragen
   ENC28J60_Interrupt0=true;

   //RestoreInterrupts();
}


void ENC28J60_Process_Interrupt()
{
   ENC28J60_Interrupt0=false;

   //Buffer des Enc's abhohlen :-)
   packet_lenght = enc28j60PacketReceive(BUFFER_SIZE, buffer);

   /*Wenn ein Packet angekommen ist, ist packet_lenght != 0*/
   if(packet_lenght)
   {

      /*Ist das Packet ein Broadcast packet, vom Typ Arp und an unsere Ip gerichtet?*/
      if(Checkbroadcast() && Checkarppackage() && Checkmyip())
      {
         Sende_ARP_Antwort(packet_lenght, buffer);
      }

      /* Ist das Packet kein Broadcast, sondern explizit an unsere mac adresse gerichtet? */
      if(Checkmymac())
      {
         /* Handelt es sich um ein ICMP Packet? ->beantworten (Ping) */
         if (buffer[IP_TYPEFIELD] == TYPE_ICMP)
            icmp(packet_lenght,buffer);

         /* Handelt es sich um ein TCP Packet fr den HTTP-Server ? */
         if ((buffer[IP_TYPEFIELD] == TYPE_TCP) && (buffer[TCP_PORT_L] == HTTP_Port))
            TCP_Verarbeite_Packet(packet_lenght,buffer);


         /* Handelt es sich um ein UDP Packet, das auf Port 85 reinkommt? */
         if(buffer[IP_TYPEFIELD] == TYPE_UDP && buffer[UDP_PORT_L] == 85)
         {
            /* Folgendes dient bloss als Anwendungsbeispiel, 
               die empfangenen Daten liegen im buffer ab adresse UDP_DATA
               und knnen natrlich auch fr ganz andere Zweche verwendet
               werden.
               TODO: - Funktion um nicht nur auf UDP Packete zu antworten,
                       sondern auch welche zu erzeugen
                     -TCP(muss ich erst noch verstehen :-) )
            */

            if(compare(&buffer[UDP_DATA], (Byte*) "test"))
            {
               Print_in_Buffer_Flash_p(&buffer[UDP_DATA], (Byte*) PSTR("Das Test Packetchen ist angekommen :-)"),TERMINATE);
               //Die Lnge 38 bezieht sich hier nur auf die Nutzdaten, headerlnge etc wird von der
               //Udp Funktion natrlich selbst bernommen...
               UDP_Sende_Antwort(38,buffer); 
            }
         }
      }
   }
}

void enc28j60PacketSend(Word len, Byte* packet)
{
	// Set the write pointer to start of transmit buffer area
	enc28j60Write(EWRPTL, TXSTART_INIT);
	enc28j60Write(EWRPTH, TXSTART_INIT>>8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ETXNDL, (TXSTART_INIT+len));
	enc28j60Write(ETXNDH, (TXSTART_INIT+len)>>8);

	// write per-packet control byte
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);

	// copy the packet into the transmit buffer
	enc28j60WriteBuffer(len, packet);
	
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
}

Word enc28j60PacketReceive(Word maxlen, Byte* packet)
{
	Word rxstat;
	Word len;

	// check if a packet has been received and buffered
	if( !(enc28j60Read(EIR) & EIR_PKTIF) )
		return 0;
	
	// Make absolutely certain that any previous packet was discarded	
	//if( WasDiscarded == FALSE)
	//	MACDiscardRx();

	// Set the read pointer to the start of the received packet
	enc28j60Write(ERDPTL, (NextPacketPtr));
	enc28j60Write(ERDPTH, (NextPacketPtr)>>8);
	// read the next packet pointer
	NextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	NextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the receive status
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;

	// limit retrieve length
	// (we reduce the MAC-reported length by 4 to remove the CRC)
	len = MIN(len, maxlen);

	// copy the packet from the receive buffer
	enc28j60ReadBuffer(len, packet);

	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	enc28j60Write(ERXRDPTL, (NextPacketPtr));
	enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);

	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	return len;
}




